from . import ldapobject
from .ldapobject import L
from .utf_tools import to_alphabetic
from .conn import _get_connection, InvalidCredentials, ConnectionFailure

from ldap_settings import LDAP_ACCESS_SECRET

from base64 import b64encode
from os import urandom

import hashlib


class LdapEcole(ldapobject.LdapObject):
    directory = "principal"
    manager = "root"
    objectClass = ["top", "organization", "Ecole"]

    o = ldapobject.LdapRDNField()
    displayName = ldapobject.LdapStringField()

    def __str__(self):
        return "{} ({})".format(self.displayName, self.o)

    @staticmethod
    def initclass(myself):
        myself.inp = myself(o="inp", parent="dc=etu-inpt,dc=fr", displayName="INP Toulouse")
        myself.n7 = myself(o="n7", parent="dc=etu-inpt,dc=fr", displayName="ENSEEIHT")
        myself.ensat = myself(o="ensat", parent="dc=etu-inpt,dc=fr", displayName="ENSAT")
        myself.a7 = myself(o="a7", parent="dc=etu-inpt,dc=fr", displayName="ENSIACET")
        myself.enm = myself(o="enm", parent="dc=etu-inpt,dc=fr", displayName="ENM")


class LdapFiliere(ldapobject.LdapObject):
    directory = "principal"
    manager = "root"
    objectClass = ["top", "organizationalUnit", "Filiere"]

    ou = ldapobject.LdapRDNField()
    shortName = ldapobject.LdapStringField()
    displayName = ldapobject.LdapStringField()
    ecole = ldapobject.LdapForeignKeyField(LdapEcole)
    inactif = ldapobject.LdapBooleanField(required=False, default=False)

    def __str__(self):
        return "{} ({})".format(self.displayName, self.ou)


class LdapAlias(ldapobject.LdapObject):
    directory = "principal"
    user = "consult"
    manager = "manager"
    objectClass = ["top", "nisMailAlias"]

    cn = ldapobject.LdapRDNField(multivalued=True)
    rfc822MailMember = ldapobject.LdapStringField(required=False, multivalued=True)


class LdapCompte(ldapobject.LdapObject):
    directory = "principal"
    manager = "root"

    objectClass = ["top", "account", "simpleSecurityObject"]

    userid = ldapobject.LdapRDNField()
    userPassword = ldapobject.LdapStringField()


class LdapEleveNomPrenom(ldapobject.LdapObject):
    directory = "principal"
    user = "consult"
    manager = "manager"

    objectClass = ["top", "person", "organizationalPerson", "inetOrgPerson", "qmailUser", "posixAccount", "shadowAccount", "Eleve"]

    uid = ldapobject.LdapRDNField()

    # Nom de famille
    sn = ldapobject.LdapStringField()

    # Prénom
    givenName = ldapobject.LdapStringField()

    @property
    def prenom(self):
        return self.givenName

    @property
    def nom(self):
        return self.sn

class LdapEleve(LdapEleveNomPrenom):
    uid = ldapobject.LdapRDNField()

    # Infos compte INP-net
    uidNumber = ldapobject.LdapIntegerField()
    gidNumber = ldapobject.LdapIntegerField()
    loginShell = ldapobject.LdapStringField()
    homeDirectory = ldapobject.LdapStringField()

    # Prénom Nom
    cn = ldapobject.LdapStringField()

    # Pareil
    displayName = ldapobject.LdapStringField()

    # Champs de recherche (sans accents)
    snSearch = ldapobject.LdapStringField()
    givenNameSearch = ldapobject.LdapStringField()

    # Informations personnelles (obligatoires)
    genre = ldapobject.LdapIntegerField()
    birthdate = ldapobject.LdapDateField(required=False)

    # Informations école/AE/INP-net
    promo = ldapobject.LdapIntegerField()
    ecole = ldapobject.LdapForeignKeyField(LdapEcole)
    filiere = ldapobject.LdapForeignKeyField(LdapFiliere)
    inscritAE = ldapobject.LdapBooleanField()
    inscritFrappe = ldapobject.LdapBooleanField()
    loginTP = ldapobject.LdapStringField()
    inscritPassVieEtudiant = ldapobject.LdapBooleanField(required=False)
    finPassVieEtudiant = ldapobject.LdapIntegerField(required=False)

    # Informations personnelles
    nickname = ldapobject.LdapStringField(required=False)
    aim = ldapobject.LdapStringField(required=False, multivalued=True)
    icq = ldapobject.LdapStringField(required=False, multivalued=True)
    googleTalk = ldapobject.LdapStringField(required=False, multivalued=True)
    jabber = ldapobject.LdapStringField(required=False, multivalued=True)
    msn = ldapobject.LdapStringField(required=False, multivalued=True)
    skype = ldapobject.LdapStringField(required=False, multivalued=True)
    yahoo = ldapobject.LdapStringField(required=False, multivalued=True)
    mailAnnexe = ldapobject.LdapStringField(required=False, multivalued=True)
    personalWebsite = ldapobject.LdapStringField(required=False, multivalued=True)

    # Adresse personnelle
    postalAddress = ldapobject.LdapStringField(required=False)
    parentsAddress = ldapobject.LdapStringField(required=False)

    # Téléphone
    mobile = ldapobject.LdapStringField(required=False)
    homePhone = ldapobject.LdapStringField(required=False)

    # Parrain de l'élève
    uidParrain = ldapobject.LdapStringField(required=False)

    # Numéro de carte de l'élève
    card = ldapobject.LdapStringField(required=False)

    # Adresse mail sur galileo (login@bde.ecole.fr)
    mail = ldapobject.LdapStringField()

    # Autres addresses mail sur Galileo (notamment prenom.nom@inpt.fr)
    mailLocal = ldapobject.LdapStringField(required=False, multivalued=True)

    # Adresse mail vers laquelle la précédente redirige
    mailForwardingAddress = ldapobject.LdapStringField(required=False)

    # Adresse mail attribuée par l’école
    mailEcole = ldapobject.LdapStringField()

    # Indique si le site perso de l’utilisateur est utilisé
    hasWebsite = ldapobject.LdapBooleanField(default=False)

    def __init__(self, *args, **kwargs):
        if 'prenom' in kwargs and 'nom' in kwargs:
            prenom = kwargs['prenom']
            nom = kwargs['nom']

            kwargs['displayName'] = kwargs['cn'] = "{} {}".format(prenom, nom)
            kwargs['sn'] = nom
            kwargs['givenName'] = prenom
            kwargs['snSearch'] = to_alphabetic(nom)
            kwargs['givenNameSearch'] = to_alphabetic(prenom)

        super(LdapEleve, self).__init__(*args, **kwargs)

    def login(self, passwd):
        """
        Tente de connecter l’utilisateur avec le mot de passe spécifié.
        Si l’opération réussit, renvoie la connexion, None sinon
        """

        try:
            return _get_connection(LDAP_ACCESS_SECRET[self.directory]['uri'], uid=self.dn, password=passwd)
        except InvalidCredentials:
            return None

        return connection

    def try_login(self, passwd):
        """
        Tente de connecter l’utilisateur avec le mot de passe spécifié.
        Si l’opération réussit, renvoie True, False sinon
        """

        result = self.login(passwd)

        if not result:
            return False

        result.unbind()

        return True

    def get_ecole(self):
        """ Indique le nom commun de l’école de l’élève : n7, ensat ou a7. """

        return self.ecole.dn.split(",", 1)[0][2:]

    def get_groupes(self):
        """ Indique les groupes de l’élève """

        return LdapGroupe.objects.filter(memberUid=self.uid)

    def get_clubs(self, ecole=None):
        """ Indique les clubs de l’élève """

        if ecole:
            return LdapClub.objects.filter(memberUid=self.uid, ecole=ecole)

        return LdapClub.objects.filter(memberUid=self.uid)

    def is_superuser(self):
        """ Indique si l’utilisateur est un administrateur (admin-inp). """

        return bool(LdapPosixGroup.objects.filter(cn="admin-inp",
            memberUid=self.uid))

    def is_staff(self):
        """ Indique si l utilisateur est un administrateur de son ecole (admin-{n7,ensat,..}) """

        return bool(LdapPosixGroup.objects.filter((L(cn="admin-" + self.get_ecole()) | L(cn="admin-inp")) & L(memberUid=self.uid)))

    def __str__(self):
        return self.cn


class LdapEleveAvecPhoto(LdapEleve):
    uid = ldapobject.LdapRDNField()
    jpegPhoto = ldapobject.LdapImageField(required=False)


class LdapEleveAvecPwd(LdapEleve):
    user = "manager"
    manager = "manager"

    uid = ldapobject.LdapRDNField()
    userPassword = ldapobject.LdapStringField()

    def change_password(self, passwd):
        """
        Change le mot de passe de l’utilisateur.
        """

        salt = urandom(4)
        sha_hash = hashlib.sha1(passwd.encode('utf8'))
        sha_hash.update(salt)

        self.userPassword = "{SSHA}" + b64encode(sha_hash.digest() + salt).decode('utf8')
        self.save()


class LdapEleveAvecPhotoEtPwd(LdapEleveAvecPwd):
    uid = ldapobject.LdapRDNField()
    jpegPhoto = ldapobject.LdapImageField(required=False)


class LdapPosixGroup(ldapobject.LdapObject):
    directory = "principal"
    manager = "root"
    objectClass = ["top", "posixGroup"]

    cn = ldapobject.LdapRDNField()
    gidNumber = ldapobject.LdapIntegerField()

    memberUid = ldapobject.LdapStringField(required=False, multivalued=True)


class LdapGroupe(LdapPosixGroup):
    objectClass = ["top", "posixGroup", "Groupe"]

    user = "consult"
    manager = "manager"

    cn = ldapobject.LdapRDNField()
    displayName = ldapobject.LdapStringField()

    # Indique si le site perso du groupe est utilisé
    hasWebsite = ldapobject.LdapBooleanField(default=False)

    domainesAssocies = ldapobject.LdapStringField(required=False, multivalued=True)

    @property
    def id_ecole(self):
        return self.cn.rsplit('-', 1)[1]

    @property
    def ident(self):
        return self.cn.rsplit('-', 1)[0]

    def __str__(self):
        return self.displayName


class LdapClub(LdapGroupe):
    objectClass = ["top", "posixGroup", "Groupe", "Club"]

    cn = ldapobject.LdapRDNField()

    president = ldapobject.LdapStringField()
    vicePresident = ldapobject.LdapStringField(required=False, multivalued=True)
    tresorier = ldapobject.LdapStringField(multivalued=True)
    secretaire = ldapobject.LdapStringField(required=False, multivalued=True)

    activite = ldapobject.LdapStringField()
    typeClub = ldapobject.LdapStringField(required=False)

    ecole = ldapobject.LdapForeignKeyField(LdapEcole)

    anneePassation = ldapobject.LdapIntegerField()

    mailAlias = ldapobject.LdapStringField(required=False, multivalued=True)
    local = ldapobject.LdapStringField(required=False)


class LdapCompteCri(ldapobject.LdapObject):
    directory = "cri"
    manager = "root"
    objectClass = ["top", "person"]

    cache_in_pool = False

    # Infos compte INP-net
    uid = ldapobject.LdapRDNField()
    givenName = ldapobject.LdapStringField()
    sn = ldapobject.LdapStringField()
    mail = ldapobject.LdapStringField(required=False, multivalued=True)
    # In theory this should be a reference to another group but it is easier to just consider them as strings
    groups = ldapobject.LdapStringField(required=False, multivalued=True)

    @property
    def premier_mail(self):
        if self.mail:
            return self.mail[0]
        return None
